// 2D-KS $Id: Range Counting Tree
// Author:  Raul H. C. Lopes

/**********************************************************************************
 * Package: 2D-KS                                                                 *
 *                                                                                *
 * Description:                                                                   *
 *      Range Counting Tree                                                       *
 *                                                                                *
 * Author (alphabetical):                                                         *
 *      Raul H. Cardoso Lopes  - Brunel Unviversity                               *
 *                               Federal University of Espirito Santo             *
 *          http://www.inf.ufes.br/~/raulh                                        *
 *          email: rlopes at cern dot ch                                          *
 *                 rhclopes at yahoo dot com dot br                               *
 *                                                                                *
 * Copyright (c) 2007:                                                            *
 *      Brunel University                                                         *
 *                                                                                *
 * This program is free software; you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by           *  
 * the Free Software Foundation; either version 2 of the License, or (at          *
 * your option) any later version.                                                * 
 *                                                                                *
 * This program is distributed in the hope that it will be useful, but            *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                     *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU              *
 * General Public License for more details.                                       *
 *                                                                                *
 *                                                                                *
 **********************************************************************************/

#include "point.h"

typedef struct ysplit ysplit_t;
struct ysplit
{
  point_t *ptref;
  int lydisp,
      rydisp;
  int lcount;
};

// rct_inode:
//  a node in range counting tree

typedef struct rct_inode rct_inode_t;
struct rct_inode
{
 point_t *medref;
 rct_inode_t *left;
 rct_inode_t *right;
 ysplit_t *yvec;
 int nysplit;
};

#define inode_median(r) r->medref

typedef struct rc_tree rc_tree_t;
struct rc_tree
{
  point_t *points;
  int npoints;
  rct_inode_t *tree;
  coord_t xmax;
  coord_t ymax;
};

ysplit_t* mk_ysplit(point_t *ptref, int lydisp, int rydisp, int lcount);

// rct_query:
//  rct: range counting tree built by mk_rc_tree
//  xr: x-coordinate of upper right corner of a bounding box
//  yt: y-coordinate of upper right corner of a bounding box
//  COMPUTES: number of points in the box (two-sided query) defined by xr, yt
//
int rct_query(rc_tree_t * rct, coord_t xr, coord_t yt);

// mk_rc_tree: 
//  pts: array of points
//  n: number of points in array
//  RETURNS: range counting tree
//
//
rc_tree_t * mk_rc_tree(point_t *pts, int n);

//# vi:sw=2 ts=2 sm et cindent backspace=eol,start,indent lines=35 columns=120
